@@ -36,6 +36,7 @@ def validate_calendar(cal, strict=False):
     """Validate a calendar object.
 
     :param cal: Calendar object
+    :return: iterator over error messages
     """
     for error in validate_component(cal, strict=strict):
         yield error
@@ -210,17 +211,27 @@ class ICalendarFile(File):
         cal = self.calendar
         # TODO(jelmer): return the list of errors to the caller
         if cal.is_broken:
-            raise InvalidFileContents(self.content_type, self.content)
-        if list(validate_calendar(cal, strict=False)):
-            raise InvalidFileContents(self.content_type, self.content)
+            raise InvalidFileContents(
+                self.content_type, self.content,
+                "Broken calendar file")
+        errors = list(validate_calendar(cal, strict=False))
+        if errors:
+            raise InvalidFileContents(
+                self.content_type, self.content,
+                ", ".join(errors))
+
+    def normalized(self):
+        """Return a normalized version of the file."""
+        return [self.calendar.to_ical()]
 
     @property
     def calendar(self):
         if self._calendar is None:
             try:
                 self._calendar = Calendar.from_ical(b''.join(self.content))
-            except ValueError:
-                raise InvalidFileContents(self.content_type, self.content)
+            except ValueError as e:
+                raise InvalidFileContents(
+                    self.content_type, self.content, str(e))
         return self._calendar
 
     def describe_delta(self, name, previous):
